@@ -1,4 +1,4 @@
-<TeXmacs|1.99.4>
+<TeXmacs|1.99.5>
 
 <style|<tuple|source|english>>
 
@@ -149,6 +149,32 @@
       <arg|body>
     </with>
   </macro>>
+
+  <assign|wide-fragment|<macro|body|<with|wpar-length|<plus|1par|0cm>|<arg|body>>>>
+
+  <assign|wide-quote-env|<\macro|body>
+    <\quote-env>
+      <arg|body>
+    </quote-env>
+  </macro>>
+
+  <assign|left-quote-env|<\macro|body>
+    <\padded>
+      <\indent-both|<value|quote-left-indentation>|0fn>
+        <with|par-first|0fn|par-par-sep|<value|quote-interparagraph>|<arg|body>>
+      </indent-both>
+    </padded>
+  </macro>>
+
+  <assign|padded-indent|<\macro|body>
+    <\padded>
+      <\with|par-first|0fn|par-par-sep|<value|quote-interparagraph>>
+        <\indent>
+          <arg|body>
+        </indent>
+      </with>
+    </padded>
+  </macro>>
 </body>
 
 <\initial>
