@@ -80,6 +80,7 @@
     (refreshable :%1 :menu-item-list)
     (if :%1 :menu-item-list)
     (when :%1 :menu-item-list)
+    (for :%1 :%1)
     (mini :%1 :menu-item-list)
     (link :%1)
     (promise :%1)
@@ -146,6 +147,24 @@
 (define (greyed? style)
   (!= (logand style widget-style-grey) 0))
 
+(define (recursive-occurs? w t)
+  (cond ((string? t) (string-occurs? w t))
+        ((list? t) (list-or (map (cut recursive-occurs? w <>) t)))
+        (else #f)))
+
+(define (recursive-replace t w b)
+  (cond ((string? t) (string-replace t w b))
+        ((list? t) (map (cut recursive-replace <> w b) t))
+        (else t)))
+
+(define (adjust-translation s t)
+  (cond ((not (and (qt-gui?) (os-macos?))) t)
+        ((recursive-occurs? "reference" s)
+	 (recursive-replace (recursive-replace t "c" "<#441>") "e" "<#435>"))
+        ((recursive-occurs? "onfigur" s)
+	 (recursive-replace t "o" "<#43E>"))
+        (else t)))
+
 (define (make-menu-label p style . opt)
   "Make widget for menu label @p."
   ;; Possibilities for p:
@@ -163,7 +182,7 @@
   (let ((tt? (and (nnull? opt) (car opt)))
         (col (color (if (greyed? style) "dark grey" "black"))))
     (cond ((translatable? p)            ; "text"
-           (widget-text (translate p) style col #t))
+           (widget-text (adjust-translation p (translate p)) style col #t))
           ((tuple? p 'balloon 2)        ; (balloon <label> "balloon text")
            (make-menu-label (cadr p) style tt?))
           ((tuple? p 'extend)           ; (extend <label> . ws)
@@ -204,7 +223,7 @@
 
 (define (make-menu-group s style)
   "Make @(group :string?) menu item."
-  (widget-menu-group (translate s) style))
+  (widget-menu-group (adjust-translation s (translate s)) style))
 
 (define (make-menu-text s style)
   "Make @(text :string?) menu item."
@@ -571,6 +590,13 @@
                                   (+ widget-style-inert widget-style-grey)))))
       (make-menu-items-list items new-style bar?))))
 
+(define (make-menu-for p style bar?)
+  "Make @(for :%1 :%1) menu items."
+  (with (tag gen-func vals-promise) p
+    (let* ((vals (vals-promise))
+           (items (append-map gen-func vals)))
+      (make-menu-items-list items style bar?))))
+
 (define (make-menu-mini p style bar?)
   "Make @(mini :%1 :menu-item-list) menu items."
   (with (tag pred? . items) p
@@ -715,6 +741,8 @@
       ,(lambda (p style bar?) (make-menu-if p style bar?)))
   (when (:%1 :*)
         ,(lambda (p style bar?) (make-menu-when p style bar?)))
+  (for (:%1 :%1)
+        ,(lambda (p style bar?) (make-menu-for p style bar?)))
   (mini (:%1 :*)
         ,(lambda (p style bar?) (make-menu-mini p style bar?)))
   (promise (:%1)
@@ -745,6 +773,13 @@
         (cons* 'when #t (menu-expand-list items))
         (cons* 'when #f (replace-procedures items)))))
 
+(define (menu-expand-for p)
+  "Expand menu @p generated using for loop."
+  (with (tag gen-func vals-promise) p
+    (let* ((vals (vals-promise))
+           (items (append-map gen-func vals)))
+      (menu-expand-list items))))
+
 (define (menu-expand-mini p)
   "Expand mini menu @p."
   (with (tag pred? . items) p
@@ -878,6 +913,7 @@
   (ink ,replace-procedures)
   (if ,menu-expand-if)
   (when ,menu-expand-when)
+  (for ,menu-expand-for)
   (mini ,menu-expand-mini)
   (promise ,menu-expand-promise)
   (refresh ,replace-procedures)
@@ -900,28 +936,54 @@
   (:argument style "menu style")
   ((wrap-catch make-menu-main) p style))
 
-(tm-define (top-window menu-promise name . opt-quit)
-  (:interactive #t)
-  (let* ((win (alt-window-handle))
-         (qqq (lambda () (if (nnull? opt-quit) ((car opt-quit)))))
-         (qui (object->command (lambda () (qqq) (alt-window-delete win))))
-         (men (menu-promise))
-         (scm (list 'vertical men))
-         (wid (make-menu-widget scm 0)))
-    (alt-window-create-quit win wid name qui)
-    (alt-window-show win)))
+(define (decode-options opts)
+  (let* ((bufs (list))
+         (qcmd noop))
+    (while (and (pair? opts) (url? (car opts)))
+      (set! bufs (cons (car opts) bufs))
+      (set! opts (cdr opts)))
+    (when (pair? opts)
+      (set! qcmd (car opts))
+      (set! opts (cdr opts)))
+    (list bufs qcmd)))
 
-(tm-define (dialogue-window menu-promise cmd name . opt-quit)
+(define window-deleters (make-ahash-table))
+
+(define (make-window-deleter win bufs)
+  (for-each (lambda (buf)
+              (and-with old-del (ahash-ref window-deleters buf) (old-del)))
+            bufs)
+  (with del
+      (lambda ()
+        (for-each (lambda (buf) (ahash-remove! window-deleters buf)) bufs)
+        (alt-window-delete win))
+    (for-each (lambda (buf) (ahash-set! window-deleters buf del)) bufs)
+    del))
+
+(tm-define (top-window menu-promise name . opts)
   (:interactive #t)
-  (let* ((win (alt-window-handle))
-         (qqq (lambda () (if (nnull? opt-quit) ((car opt-quit)))))
-         (qui (object->command (lambda () (qqq) (alt-window-delete win))))
-         (lbd (lambda x (apply cmd x) (alt-window-delete win)))
-         (men (menu-promise lbd))
-         (scm (list 'vertical men))
-         (wid (make-menu-widget scm 0)))
-    (alt-window-create-quit win wid name qui)
-    (alt-window-show win)))
+  (with (bufs qqq) (decode-options opts)
+    (let* ((win (alt-window-handle))
+           (del (make-window-deleter win bufs))
+           (qui (object->command (lambda () (qqq) (del))))
+           (men (menu-promise))
+           (scm (list 'vertical men))
+           (wid (make-menu-widget scm 0)))
+      (alt-window-create-quit win wid name qui)
+      (alt-window-show win))))
+
+(tm-define (dialogue-window menu-promise cmd name . opts)
+  (:interactive #t)
+  (with (bufs qqq) (decode-options opts)
+    (let* ((win (alt-window-handle))
+           (del (make-window-deleter win bufs))
+           (qui (object->command (lambda () (qqq) (del))))
+           (lbd (lambda x (apply cmd x) (del)))
+           (men (menu-promise lbd))
+           (scm (list 'vertical men))
+           (wid (make-menu-widget scm 0)))
+      (alt-window-create-quit win wid name qui)
+      (alt-window-show win))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Other top-level windows
@@ -962,3 +1024,47 @@
       (with p (lambda (com) (widget-color-picker com #f proposals))
         (with cmd* (lambda (t) (when t (cmd (tm->stree t))))
           (interactive-window p cmd* "Choose background")))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Reporting errors of system commands
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-widget ((system-error-widget cmd out err) done)
+  (padded
+    (resize ("300px" "600px" "1200px") ("275px" "400px" "600px")
+      (centered (bold (text "Input command")))  
+      (scrollable
+	(for (x (string-decompose cmd "\n"))
+	  (hlist // (text x) >>)))
+      ===
+      (centered (bold (text "Standard Output")))
+      (scrollable
+	(for (x (string-decompose out "\n"))
+	  (hlist // (text x) >>)))
+      ===
+      (centered (bold (text "Error output")))
+      (scrollable
+	(for (x (string-decompose err "\n"))
+	  (hlist // (text x) >>)))
+      ===
+      (bottom-buttons >> ("Ok" (done))))))
+
+(tm-define (report-system-error win-name cmd out err)
+  (:synopsis "Display command @cmd with its standard outputs @out and @err")
+  (when (list? cmd) (set! cmd (string-recompose cmd " ")))
+  (set! out (utf8->cork out))
+  (set! err (utf8->cork err))
+  (dialogue-window (system-error-widget cmd out err) noop win-name)
+  #f)
+
+(tm-widget ((message-widget msg) done)
+  (padded
+    (centered (text msg))
+    ===
+    (centered
+      (explicit-buttons
+        ("Ok" (done))))))
+
+(tm-define (show-message msg title)
+  (:interactive #t)
+  (dialogue-window (message-widget msg) noop title))
